MAINNET_API_HTTP = "https://virginia.solana.dex.blxrbdn.com"
MAINNET_API_WS = "wss://virginia.solana.dex.blxrbdn.com/ws"
MAINNET_API_GRPC_HOST = "virginia.solana.dex.blxrbdn.com"
MAINNET_API_GRPC_PORT = 443

TESTNET_API_HTTP = (
    "http://serum-nlb-5a2c3912804344a3.elb.us-east-1.amazonaws.com/"
)
TESTNET_API_WS = (
    "ws://serum-nlb-5a2c3912804344a3.elb.us-east-1.amazonaws.com/ws"
)
TESTNET_API_GRPC_HOST = "serum-nlb-5a2c3912804344a3.elb.us-east-1.amazonaws.com"
TESTNET_API_GRPC_PORT = 80

DEVNET_API_HTTP = (
    "http://serum-nlb-53baf45ef9775263.elb.us-east-1.amazonaws.com/"
)
DEVNET_API_WS = "ws://serum-nlb-53baf45ef9775263.elb.us-east-1.amazonaws.com/ws"
DEVNET_API_GRPC_HOST = "serum-nlb-53baf45ef9775263.elb.us-east-1.amazonaws.com"
DEVNET_API_GRPC_PORT = 80

LOCAL_API_HTTP = "http://127.0.0.1:9000"
LOCAL_API_WS = "ws://127.0.0.1:9000/ws"
LOCAL_API_GRPC_HOST = "127.0.0.1"
LOCAL_API_GRPC_PORT = 9000
